package com.emperador.exoplayer.service

import android.app.*
import android.content.Intent
import android.net.Uri
import android.os.IBinder
import android.util.Log
import com.google.android.exoplayer2.extractor.DefaultExtractorsFactory
import com.google.android.exoplayer2.source.ProgressiveMediaSource
import com.google.android.exoplayer2.source.hls.HlsMediaSource
import com.google.android.exoplayer2.trackselection.AdaptiveTrackSelection
import com.google.android.exoplayer2.trackselection.DefaultTrackSelector
import com.google.android.exoplayer2.upstream.DefaultBandwidthMeter
import com.google.android.exoplayer2.util.Util.getUserAgent
import com.google.android.exoplayer2.upstream.DefaultDataSourceFactory
import android.content.BroadcastReceiver
import android.content.Context
import android.content.IntentFilter
import com.google.android.exoplayer2.*
import android.media.session.MediaSession
import android.os.Build
import androidx.annotation.RequiresApi
import android.os.Bundle
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.media.MediaDescription
import android.media.MediaMetadata
import android.media.session.MediaController
import android.media.session.PlaybackState
import androidx.annotation.NonNull
import androidx.core.app.NotificationCompat
import androidx.core.app.NotificationManagerCompat
import com.google.android.exoplayer2.ui.PlayerNotificationManager.ACTION_NEXT
import io.flutter.plugin.common.MethodChannel


class MusicService : Service(), Player.EventListener {


    private var player: SimpleExoPlayer? = null

    companion object {

        const val TYPE_PROGRESSIVE = 0
        const val TYPE_HLS = 1

        const val ACTION_PLAY = "play"
        const val ACTION_STOP = "stop"
        const val ACTION_PAUSE = "pause"

        const val PLAYBACK_STATE = "state_change"
    }


    private val serviceReceiver = object : BroadcastReceiver() {

        override fun onReceive(context: Context, intent: Intent) {
            Log.i("MusicService", "broadcast receiver -> ${intent.action}")
            when (intent.action) {
                ACTION_PLAY -> {
                    startPlayer()
                }
                ACTION_PAUSE -> {
                    pausePlayer()
                }
                ACTION_STOP -> {
                    player?.stop()
                    player?.release()
                }
            }
        }
    }

    override fun onCreate() {
        super.onCreate()

        registerReceiver(serviceReceiver, IntentFilter(ACTION_PLAY))
        registerReceiver(serviceReceiver, IntentFilter(ACTION_STOP))
        registerReceiver(serviceReceiver, IntentFilter(ACTION_PAUSE))

        val i = Intent()
        i.action = PLAYBACK_STATE
        i.putExtra("state", "SERVICE_START")
        sendBroadcast(i)
    }

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        Log.d("MusicService", "MusicService")

        val url = intent!!.getStringExtra("url")
        val type = intent.getIntExtra("type", -1)

        val bandwidthMeter = DefaultBandwidthMeter()
        val extractorsFactory = DefaultExtractorsFactory()

        val trackSelectionFactory = AdaptiveTrackSelection.Factory(bandwidthMeter)

        val trackSelector = DefaultTrackSelector(trackSelectionFactory)

        val defaultBandwidthMeter = DefaultBandwidthMeter()

        val dataSourceFactory = DefaultDataSourceFactory(this,
                getUserAgent(this, "mediaPlayerSample"), defaultBandwidthMeter)


        val uri = Uri.parse(url)

        Log.i("MusicService", "type $type / uri:$uri")

        val mediaSource = when (type) {
            TYPE_PROGRESSIVE -> ProgressiveMediaSource.Factory(dataSourceFactory).createMediaSource(uri)
            TYPE_HLS -> HlsMediaSource.Factory(dataSourceFactory).createMediaSource(uri)
            else -> ProgressiveMediaSource.Factory(dataSourceFactory).createMediaSource(uri)
        }


        player = ExoPlayerFactory.newSimpleInstance(applicationContext, trackSelector)
        player?.addListener(this)
        player?.prepare(mediaSource)

        buildNotification(type.toString(), url)

        return START_NOT_STICKY
    }

    private fun buildNotification(title: String, detail: String) {
        createNotificationChannel()
        val builder = NotificationCompat.Builder(this, "99")
                .setSmallIcon(R.drawable.exo_icon_play)
                .setContentTitle(title)
                .setContentText(detail)
                .setAutoCancel(false)
                .setPriority(NotificationCompat.PRIORITY_DEFAULT)

        with(NotificationManagerCompat.from(this)) {
            notify(1234, builder.build())
        }
    }

    private fun removeNotification() {
        val notificationManager = getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
        notificationManager.cancel(1234)
    }

    private fun createNotificationChannel() {
        // Create the NotificationChannel, but only on API 26+ because
        // the NotificationChannel class is new and not in the support library
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val name = "chanel name"
            val descriptionText = "chanel description"
            val importance = NotificationManager.IMPORTANCE_DEFAULT
            val channel = NotificationChannel("99", name, importance).apply {
                description = descriptionText
            }
            // Register the channel with the system
            val notificationManager: NotificationManager =
                    getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
            notificationManager.createNotificationChannel(channel)
        }
    }

    override fun onPlayerStateChanged(playWhenReady: Boolean, playbackState: Int) {

        val i = Intent()
        i.action = PLAYBACK_STATE

        when (playbackState) {
            ExoPlayer.STATE_IDLE -> {
                i.putExtra("state", "STATE_IDLE")
            }
            ExoPlayer.STATE_READY -> {
                i.putExtra("state", "STATE_READY")
            }
            ExoPlayer.STATE_BUFFERING -> {
                i.putExtra("state", "STATE_BUFFERING")
            }
            ExoPlayer.STATE_ENDED -> {
                i.putExtra("state", "STATE_ENDED")
            }
        }

        sendBroadcast(i)


    }

    override fun onPlayerError(error: ExoPlaybackException?) {
        val i = Intent()
        i.action = PLAYBACK_STATE
        i.putExtra("state", "STATE_ERROR")
        sendBroadcast(i)
    }

    private fun pausePlayer() {
        player?.playWhenReady = false
        player?.playbackState

        val i = Intent()
        i.action = PLAYBACK_STATE
        i.putExtra("state", "STATE_PAUSED")
        sendBroadcast(i)
    }

    private fun startPlayer() {
        player?.playWhenReady = true
        player?.playbackState


        val i = Intent()
        i.action = PLAYBACK_STATE
        i.putExtra("state", "STATE_PLAYING")
        sendBroadcast(i)
    }

    override fun onBind(p0: Intent?): IBinder? {
        return null
    }


    override fun onDestroy() {
        unregisterReceiver(serviceReceiver)
        removeNotification()

        val i = Intent()
        i.action = PLAYBACK_STATE
        i.putExtra("state", "STATE_PAUSED")
        sendBroadcast(i)

        val i2 = Intent()
        i2.action = PLAYBACK_STATE
        i2.putExtra("state", "SERVICE_DESTROY")
        sendBroadcast(i2)


        super.onDestroy()
    }

    override fun onTaskRemoved(rootIntent: Intent?) {
        onDestroy()
        // super.onTaskRemoved(rootIntent)
    }

}

